import java.util.*;

/**
 *  Elements, Isotpes, sigh.
 */


class Element {
  final public String symbol;
  final public int Z;
  //final public double atomicW; // official atomic weight (we do not use it)
  final public double monoW; // mass of "selected" isotope
  final public Isotope[] isotopes;
  private double[] abundanceladder;
  //Note, the "selected" isotope must be the first in the data arrays!
  Element(String symbol, int Z, int[] Aarray, double[][] WAarray) {
    this.symbol = symbol;
    this.Z = Z;
    this.monoW = WAarray[0][0];
    isotopes = new Isotope[Aarray.length];
    abundanceladder = new double[Aarray.length];
    double lastladder = 0.0;
    for (int i=0; i<Aarray.length; i++) {
      isotopes[i] = new Isotope(Aarray[i], WAarray[i][0], WAarray[i][1]);
      lastladder = abundanceladder[i] = WAarray[i][1] + lastladder;
    }
  }

  public String toString() {
    StringBuffer res = new StringBuffer(symbol+" ("+Z+", "+monoW+"): ");
    for (int i=0; i<isotopes.length; i++) {
      res.append(isotopes[i].toString() + " "+  abundanceladder[i]);
      res.append(" ");
    }
    return res.toString();
  }

  public Isotope randomIsotope(Random rnd) {
    double r = rnd.nextDouble();
    int i=0;
    //I am not sure this is the most efficient algorithm from an
    //asymptotics point of view, but in the current setting, there are
    //at most 3 isotopes and abundanceladder[0] is very close to
    //1.0. Therefore this is probably as efficient as it gets here.
    while (i<abundanceladder.length-1 && abundanceladder[i]<r)
      i++;
    return isotopes[i];
  }
  
  class Isotope {
    final int A;
    final double W; 
    final double abundance;
    final String isosymbol;
    Isotope(int A, double W, double abundance) {
      this.A = A;
      this.W = W;
      this.abundance = abundance;
      isosymbol = symbol+A;
    }
    public String toString() {
      return "("+isosymbol+", "+W+", "+abundance+")";
    }
  }

  //Info on all relevant elements for amino acids/peptides. Na is in
  //here for testing purposes.

  final static Element H = new Element("H", 1,
				       new int[] {1,2},
				       new double[][] {{1.007825 ,0.99985},
						       {2.014101 ,0.00015}});
  
  final static Element C = new Element("C", 6,
				       new int[] {12,13},
				       new double[][] {{12.00000, 0.9889},
						       {13.00335, 0.0111}});
  final static Element N = new Element("N", 7,
				       new int[] {14, 15},
				       new double[][] {{14.00307, 0.9963},
						       {15.00011, 0.0037}});
  final static Element O = new Element("O", 8,
				       new int[] {16, 17, 18},
				       new double[][] {{15.99491, 0.99763},
						       {16.99913, 0.00037},
						       {17.99916, 0.00200}});
  final static Element Na = new Element("Na", 11,
				       new int[] {23},
				       new double[][] {{22.98977, 1.0}});

  final static Element S = new Element("S", 16,
				       new int[] {32, 33, 34},
				       new double[][] {{31.97207, 0.9502},
						       {32.97146, 0.0076},
						       {33.96786, 0.0422}});

  final static int Z_LIMIT = 119; //That limit should be safe for a while

  final static Element[] elements = new Element[Z_LIMIT]; 

  private final static HashMap /* name->element*/ name2element;

  static {
    Element[] tmpelem = {H, C, N, O, Na, S};
    name2element = new HashMap();
    for (int i=0; i<tmpelem.length; i++) {
      Element elem = tmpelem[i];
      elements[elem.Z] = elem;
      name2element.put(elem.symbol, elem);
    }
  }

  public static Element get(String n) {
    return (Element)(name2element.get(n));
  }

  public static void main(String[] args) {
    for (int i=0; i<elements.length; i++)
      if (elements[i] != null)
	System.out.println(elements[i].toString());
  }

}

