
import java.awt.*;
import java.awt.geom.*;
import java.text.*;
import java.util.*;
import javax.swing.*;

/**
 * An abstract marking
 */
abstract class Mark {
  double x, y;			// location of this mark
  Peak peak;			// "nearest" peak
  double score;                 
  AbstractSequence seq; // sequence to which this belongs
  //Note: we could make this an inner class of sequence, then seq
  //would be implicit, and the design would be more clean: some
  //methods need access to acceptedL/R, which should be private to
  //Sequence. Alternatively, there could be peer methods in Sequence
  //we could call. BUT: it is used in the current undo code that the
  //Sequence reference is explicit.

  int kind;

  final static int POTENTIAL = 0;
  final static int ACCEPTED = 1;
  final static int START = 2;

  public void draw(Graphics2D g, AbstractCoordinates coor, 
		   double tolerance, boolean showErrors) {
    Color c = g.getColor();
    drawInner(g, coor, tolerance, showErrors);
    g.setColor(c);
  }

  public double error() { 
    return x - peak.x; 
  }

  abstract public void remove();

  abstract public String getAbbr();
  
  abstract void drawInner(Graphics2D g, AbstractCoordinates coor, 
			  double tolerance, boolean showErrors);
}


