
/**
 * Fundamental unit of data in Spectrum
 */
final class Peak implements Comparable {
  //  final static double BOTTOM_OFFSET = 19.018385; //bottom peak H3O
  //final static double TOP_OFFSET = -1; // top peak (from totalMass)
  //Origin aka "kind" of this peak
  final static int ORIGINAL           = 0; 
  final static int SYNTHETIC_BOTTOM   = 1; 
  final static int SYNTHETIC_TOP      = 2; 
  final static int TRANSFORMED        = 3; 
  final static int MIRRORED           = 4; 
  
  //fake original indices. We use p.orig_index>=0 as a test for
  //syntheticity
  final static int BOTTOM_OI          = -1;
  final static int TOP_OI             = -2;
  // cutoff positions for at_end test for KR
  final static double MAX_END_OFFSET  = 21.0;
  //
  double x; //mass, possibly scaled
  double y; //abundance
  final double x_orig; //original mass (should be superfluous)
  final int orig_index;
  int kind; //original, mirrored, synthetic, transformed
  IonType iontype; //null means assumed default i.e. Y
  boolean ignore; 
  //It is questionable whether the local levels should be attached to
  //individual peaks. This must be decided if we have any other use
  //for the levels than just showing them
  double la; //local abundance level
  double lb; //local background level
  //For comparing peaks
  private int id;
  private static int lastpeakid; 
  Peak(double x, double x_orig, double y, int i, int k) {
    this.x = x;
    this.y = y;
    this.x_orig = x_orig;
    this.orig_index = i;
    this.kind = k;
    id = ++lastpeakid;
  }
  //
  public int compareTo(Object o) {
    double otherx = ((Peak) o).x;
    return x>otherx ? 1 : x<otherx ? -1 : ((Peak) o).id-id ;
  }
}
