
class SeqScore implements Cloneable, Score {
  //Info on spectrum needed for computing score
  private double maxAbundance;
  private double totalMass;
  //parameters for the score/penalty formulae
  private SequencingParameters seqParms;

  SeqScore(double maxAbundance, double totalMass, 
	SequencingParameters seqParms) {
    //Remember the parameters
    this.maxAbundance = maxAbundance;
    this.totalMass = totalMass;
    this.seqParms = seqParms;
  }

  private double kr_correction(double x1, double x2, AminoAcid acid) {
    double kr_correction = 0.0;
    if (acid.abbr == "K" || acid.abbr == "R") {
      if (x1 < Peak.MAX_END_OFFSET || x2 < Peak.MAX_END_OFFSET) 
	// K or R at low end: there is a bonus!
	kr_correction = -seqParms.krpenalty;
      else if (!seqParms.allowKR) 
	// K or R in the middle of the sequence
	kr_correction = seqParms.krpenalty;
    }
    return kr_correction;
  }

  //The methods of interface Score

  public double peakscore(Peak p) {
    double rel_a = p.y/maxAbundance;
    return -seqParms.lambda*Math.log((rel_a+seqParms.nu)/rel_a);
  }

  public double acidscore(AminoAcid a, double mass_error, Peak p1, Peak p2) {
    double mean_rel_a = Math.sqrt(p1.y*p2.y)/maxAbundance;
    double rel_me = mass_error/seqParms.sigma;
    double penalty = 
      0.5*rel_me*rel_me*seqParms.nu/(mean_rel_a+seqParms.nu)
      + kr_correction(p1.x, p2.x, a);
    return 
      (1-seqParms.beta)
      + seqParms.beta*seqParms.expectedCount/totalMass 
      - seqParms.lambda*penalty;
  }

  public double stepscore(AminoAcid a, double mass_error, Peak p1, Peak p2) {
    return peakscore(p1)+acidscore(a, mass_error, p1, p2);
  }

  public double getGrass() {
    return seqParms.grass;
  }

  public double getTolerance() {
    return seqParms.tolerance;
  }

  public boolean getIndependentErrors() {
    return seqParms.independent_errors;
  }

  public boolean getAllowDuplicatePeaks() {
    return seqParms.allowDuplicatePeaks;
  }

}
